@@ -82,8 +82,6 @@ func_query_expander_postgresql(grn_ctx *ctx,
 	{
 		Datum synonymsDatum;
 		bool isNULL;
-		ArrayType *synonymsArray;
-		int i, n;
 
 		if (currentData.scan)
 			tuple = index_getnext(currentData.scan, ForwardScanDirection);
@@ -100,36 +98,61 @@ func_query_expander_postgresql(grn_ctx *ctx,
 		if (isNULL)
 			continue;
 
-		synonymsArray = DatumGetArrayTypeP(synonymsDatum);
-		n = ARR_DIMS(synonymsArray)[0];
-		if (n == 0)
-			continue;
-
-		if (ith_synonyms == 0)
-			GRN_TEXT_PUTC(ctx, expandedTerm, '(');
-		else
-			GRN_TEXT_PUTS(ctx, expandedTerm, " OR ");
-
-		for (i = 1; i <= n; i++)
+		if (currentData.synonymsAttribute->atttypid == TEXTOID)
 		{
-			Datum synonymDatum;
-			bool isNULL;
 			text *synonym;
+			synonym = DatumGetTextP(synonymsDatum);
 
-			synonymDatum = array_ref(synonymsArray, 1, &i, -1,
-									 currentData.synonymsAttribute->attlen,
-									 currentData.synonymsAttribute->attbyval,
-									 currentData.synonymsAttribute->attalign,
-									 &isNULL);
-			synonym = DatumGetTextP(synonymDatum);
-			if (i > 1)
+			if (ith_synonyms == 0)
+				GRN_TEXT_PUTC(ctx, expandedTerm, '(');
+			else
 				GRN_TEXT_PUTS(ctx, expandedTerm, " OR ");
+
 			GRN_TEXT_PUTC(ctx, expandedTerm, '(');
 			GRN_TEXT_PUT(ctx, expandedTerm,
 						 VARDATA_ANY(synonym),
 						 VARSIZE_ANY_EXHDR(synonym));
 			GRN_TEXT_PUTC(ctx, expandedTerm, ')');
 		}
+		else
+		{
+			ArrayType *synonymsArray;
+			int i, n;
+
+			synonymsArray = DatumGetArrayTypeP(synonymsDatum);
+			if (ARR_NDIM(synonymsArray) == 0)
+				continue;
+
+			n = ARR_DIMS(synonymsArray)[0];
+			if (n == 0)
+				continue;
+
+			if (ith_synonyms == 0)
+				GRN_TEXT_PUTC(ctx, expandedTerm, '(');
+			else
+				GRN_TEXT_PUTS(ctx, expandedTerm, " OR ");
+
+			for (i = 1; i <= n; i++)
+			{
+				Datum synonymDatum;
+				bool isNULL;
+				text *synonym;
+
+				synonymDatum = array_ref(synonymsArray, 1, &i, -1,
+										 currentData.synonymsAttribute->attlen,
+										 currentData.synonymsAttribute->attbyval,
+										 currentData.synonymsAttribute->attalign,
+										 &isNULL);
+				synonym = DatumGetTextP(synonymDatum);
+				if (i > 1)
+					GRN_TEXT_PUTS(ctx, expandedTerm, " OR ");
+				GRN_TEXT_PUTC(ctx, expandedTerm, '(');
+				GRN_TEXT_PUT(ctx, expandedTerm,
+							 VARDATA_ANY(synonym),
+							 VARSIZE_ANY_EXHDR(synonym));
+				GRN_TEXT_PUTC(ctx, expandedTerm, ')');
+			}
+		}
 
 		ith_synonyms++;
 
@@ -175,19 +198,21 @@ PGrnFindSynonymsAttribute(const char *tableName,
 	desc = RelationGetDescr(table);
 	for (i = 1; i <= desc->natts; i++)
 	{
-		Form_pg_attribute attribute = desc->attrs[i - 1];
+		Form_pg_attribute attribute = TupleDescAttr(desc, i - 1);
 
 		if (strlen(attribute->attname.data) != columnNameSize)
 			continue;
 		if (strncmp(attribute->attname.data, columnName, columnNameSize) != 0)
 			continue;
 
-		if (attribute->atttypid != TEXTARRAYOID)
+		if (!(attribute->atttypid == TEXTOID ||
+			  attribute->atttypid == TEXTARRAYOID))
 		{
 			ereport(ERROR,
 					(errcode(ERRCODE_INVALID_NAME),
 					 errmsg("pgroonga: query_expand: "
-							"synonyms column isn't text[] type: <%s>.<%.*s>",
+							"synonyms column isn't text type nor text[] type: "
+							"<%s>.<%.*s>",
 							tableName,
 							(int)columnNameSize, columnName)));
 		}
@@ -228,7 +253,7 @@ PGrnFindTermIndex(Relation table,
 		index = index_open(indexOID, NoLock);
 		for (i = 1; i <= index->rd_att->natts; i++)
 		{
-			const char *name = index->rd_att->attrs[i - 1]->attname.data;
+			const char *name = TupleDescAttr(index->rd_att, i - 1)->attname.data;
 			Oid opFamily;
 			StrategyNumber strategy;
 
@@ -296,7 +321,7 @@ PGrnFindTermAttributeNumber(const char *tableName,
 	desc = RelationGetDescr(table);
 	for (i = 1; i <= desc->natts; i++)
 	{
-		Form_pg_attribute attribute = desc->attrs[i - 1];
+		Form_pg_attribute attribute = TupleDescAttr(desc, i - 1);
 
 		if (strlen(attribute->attname.data) != columnNameSize)
 			continue;
@@ -397,7 +422,7 @@ pgroonga_query_expand(PG_FUNCTION_ARGS)
 	grn_expr_syntax_expand_query(ctx,
 								 VARDATA_ANY(query),
 								 VARSIZE_ANY_EXHDR(query),
-								 GRN_EXPR_SYNTAX_QUERY,
+								 PGRN_EXPR_QUERY_PARSE_FLAGS,
 								 grn_ctx_get(ctx,
 											 PGRN_EXPANDER_NAME,
 											 PGRN_EXPANDER_NAME_LENGTH),
